# %% bcmfR R-SOURCE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# __________________________________________________________________________
#
#    bcmf_bcfjoint_39.R
# __________________________________________________________________________
#

# === [] ====================
# ___ () ____________________

# /////////////////



# dotrace01 <- TRUE


# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

#/////////////////////////////////////////////
# === [BootModel_Aqueous] ====================
# ~~~ needs global DATA.Aqueous.Work
# UNTRANSFORMED
BootModel_Aqueous <- function(datacfish = DATA.Aqueous.Work, 
                              paramfit  = BCF.ParamFit,
                              testparam = TEST.Aqueous.Design,
                              numboot   = NUM.Boot,
                              numcontrol= NUM.Control.nls,
                              randseed  = 3001,
                              savefit   = TRUE) {
  
  # -- Aqueous needs testparams cwater and tdepur
  cwater <- testparam["cwater", "value"]
  tdepur <- testparam["tdepur", "value"]
  
  startfit <- paramfit
  names(startfit) <- c("fitk1", "fitk2")
  
  # -- declare arrays for boot
  fitk1arr <- numeric(numboot)
  fitk2arr <- numeric(numboot)
  
  # -- store the base run in the first line of the array
  fitk1arr[1] <- paramfit["k1"]
  fitk2arr[1] <- paramfit["k2"]
  
  # number of data points
  nrows <- length(row.names(datacfish))
  
  # declare weight array as numboot x nrows
  weightsarr <- matrix(nrow = numboot, ncol = nrows)
  weightsarr[1, ] <- 1/nrows
  
  # number of boot runs not converged set to zero
  num.notconverged <- 0
  
  # set random seed for Bayesian bootstrap
  set.seed(randseed)
  
  
  for (i in 2:numboot) {
    
    repeat {
    
      # -- draw Bayesian weights (Bayesian Bootstrap)
      bb <- runif(nrows - 1)
      bbplus <- c(0., bb, 1.)
      bbsort <- sort(bbplus)
      newweights <- diff(bbsort)
      
      # -- try nls fit UNTRANSFORMED
      fitbest.out <- try(nls(cfish.data ~ RunModel_Aqueous(time.data,
                                                           cwater, tdepur,
                                                           fitk1, fitk2),
                         data    = datacfish,
                         start   = startfit,
                         weights = newweights,
                         control = numcontrol,
                         trace   = FALSE), silent = TRUE)
      
      if (class(fitbest.out)!="try-error")  {
        
        # converged, store results (store output list as well?)
        
        parfiti    <- coef(fitbest.out)
        weightsarr[i, ] <- newweights
        
        fitk1arr[i] <- parfiti[1]
        fitk2arr[i] <- parfiti[2]
        
        break
        
      }  else  {
        
        num.notconverged <- num.notconverged + 1
        
        if (num.notconverged == numboot) stop()
        
        print(num.notconverged)
        
      }
    }
  }
  
 
  # -- put into dataframe
  nlsboot <- data.frame(k1.boot = fitk1arr, k2.boot = fitk2arr)
  
  if (savefit) {
    assign("BCF.ParamBoot", nlsboot, envir = .GlobalEnv)
  }
  
  top    <- head(nlsboot, n = 10)
  bottom <- tail(nlsboot, n = 10)
  
  print(top)
  cat("   ...")
  cat("\n")
  print(bottom)
  
  # return(nlsboot)

}
# ___ (BootModel_Aqueous) ____________________


# ///////////////////////////////////////////////////
# === [BootModel_Aqueous_BoxCox] ====================
# ~~~ needs global DATA.Aqueous.Work
# ~~~ nlstools quirk: not via args
BootModel_Aqueous_BoxCox <- function(datacfish = DATA.Aqueous.Work, 
                                     paramfit  = BCF.ParamFit.BoxCox,
                                     lambda    = BCF.ParamFit.Lambda,
                                     testparam = TEST.Aqueous.Design,
                                     lnstarter = 1e-9,
                                     numboot   = NUM.Boot,
                                     numcontrol= NUM.Control.nls,
                                     randseed  = 3001,
                                     savefit   = TRUE) {
  
  # -- Aqueous needs testparams cwater and tdepur
  cwater <- testparam["cwater", "value"]
  tdepur <- testparam["tdepur", "value"]
  
  startfit <- paramfit
  names(startfit) <- c("fitk1", "fitk2")
  

  fitk1arr <- numeric(numboot)
  fitk2arr <- numeric(numboot)
  
  # -- store the base run in the first line of the array
  fitk1arr[1] <- paramfit["k1"]
  fitk2arr[1] <- paramfit["k2"]
  
  # number of data points
  nrows <- length(row.names(datacfish))
  
  # declare weight array as numboot x nrows
  weightsarr <- matrix(nrow = numboot, ncol = nrows)
  weightsarr[1, ] <- 1/nrows
  
  # number of boot runs not converged set to zero
  num.notconverged <- 0
  
  # set random seed for Bayesian bootstrap
  set.seed(randseed)
  
  
  for (i in 2:numboot) {
    
    repeat {
    
      # -- draw Bayesian weights (Bayesian Bootstrap)
      bb <- runif(nrows - 1)
      bbplus <- c(0., bb, 1.)
      bbsort <- sort(bbplus)
      newweights <- diff(bbsort)  
    
      # -- try nls fit BOXCOX
      fitbestbc.out <- try(nls(bc.cfish.data ~ car::bcPower(RunModel_Aqueous(time.data,
                                                              cwater, tdepur,
                                                              fitk1, fitk2) + lnstarter,
                                                              lambda),
                               data    = datacfish,
                               start   = startfit,
                               weights = newweights,
                               control = numcontrol,
                               trace   = FALSE), silent = TRUE)
      
      if(class(fitbestbc.out)!="try-error")  {
        
        # converged, store results (store output list as well?)
        
        parfiti <- coef(fitbestbc.out)
        
        weightsarr[i, ] <- newweights
        
        fitk1arr[i] <- parfiti[1]
        fitk2arr[i] <- parfiti[2]
        
        break
        
      }  else  {
        
        num.notconverged <- num.notconverged + 1
        
        if (num.notconverged == numboot) stop()
        
        print(num.notconverged)
        
      }
    }
  }

  # -- put into dataframe
  nlsboot <- data.frame(k1.boot.bc = fitk1arr, k2.boot.bc = fitk2arr)
  
  if (savefit) {
    assign("BCF.ParamBoot.BoxCox", nlsboot, envir = .GlobalEnv)
  }
  
  top    <- head(nlsboot, n = 10)
  bottom <- tail(nlsboot, n = 10)
  
  print(top)
  cat("   ...")
  cat("\n")
  print(bottom)
  
  # return(nlsboot)
}
# ___ (BootModel_Aqueous_BoxCox) ____________________


# ///////////////////////////////////////////////
# === [BootModel_Aqueous_Ln] ====================
# ~~~ Rubin's Bayesian Bootstrap, homemade
# ~~~ assumes ln.cfish.data to be precalculated in DATA.Aqueous.Work
BootModel_Aqueous_Ln <- function(datacfish = DATA.Aqueous.Work, 
                                 paramfit  = BCF.ParamFit.Ln,
                                 testparam = TEST.Aqueous.Design,
                                 lnstarter = 1e-9,
                                 numboot   = NUM.Boot,
                                 numcontrol= NUM.Control.nls,
                                 randseed  = 3001,
                                 savefit   = TRUE) {
  
  # -- Aqueous needs testparams cwater and tdepur
  cwater <- testparam["cwater", "value"]
  tdepur <- testparam["tdepur", "value"]
  
  startfit <- paramfit
  names(startfit) <- c("fitk1", "fitk2")
  
  # -- declare arrays for boot
  fitk1arr <- numeric(numboot)
  fitk2arr <- numeric(numboot)
  
  # -- store the base run in the first line of the array
  fitk1arr[1] <- paramfit["k1"]
  fitk2arr[1] <- paramfit["k2"]
  
  # number of data points
  nrows <- length(row.names(datacfish))
  
  # declare weight array as numboot x nrows
  weightsarr <- matrix(nrow = numboot, ncol = nrows)
  weightsarr[1, ] <- 1/nrows
  
  # number of boot runs not converged set to zero
  num.notconverged <- 0
  
  # set random seed for Bayesian bootstrap
  set.seed(randseed)
  
  
  for (i in 2:numboot) {
    
    repeat {
    
      # -- draw Bayesian weights (Bayesian Bootstrap)
      bb <- runif(nrows - 1)
      bbplus <- c(0., bb, 1.)
      bbsort <- sort(bbplus)
      newweights <- diff(bbsort)
      
      # -- try nls fit LN-TRANSFORMED
      fitbestln.out <- try(nls(ln.cfish.data ~ log(RunModel_Aqueous(time.data, cwater, tdepur,
                                                                fitk1, fitk2) + lnstarter),
                           data    = datacfish,
                           start   = startfit,
                           weights = newweights,
                           control = numcontrol,
                           trace   = FALSE), silent = TRUE)
      
      if (class(fitbestln.out)!="try-error") {
        
        # converged, store results (store output list as well?)
        
        parfiti    <- coef(fitbestln.out)
        weightsarr[i, ] <- newweights
        
        fitk1arr[i] <- parfiti[1]
        fitk2arr[i] <- parfiti[2]
        
        break
        
      }  else  {
        
        num.notconverged <- num.notconverged + 1
        
        if (num.notconverged == numboot) stop()
        
        print(num.notconverged)
        
      }
    }
  }
  
  # -- put into dataframe
  nlsboot <- data.frame(k1.boot.ln = fitk1arr, k2.boot.ln = fitk2arr)
  
  
  if (savefit) {
    assign("BCF.ParamBoot.Ln", nlsboot, envir = .GlobalEnv)
  }
  
  top    <- head(nlsboot, n = 10)
  bottom <- tail(nlsboot, n = 10)
  
  print(top)
  cat("   ...")
  cat("\n")
  print(bottom)
  
  # return(nlsboot)
}
# ___ (BootModel_Aqueous_Ln) ____________________



# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@


# ////////////////////////////////////////////
# === [ ModelDiagnostics_Aqueous] ====================
# --- package nlstools
 ModelDiagnostics_Aqueous <- function(modelfit = BCF.ModelFit) {
  nlsresun <- nlstools::nlsResiduals(modelfit)
  plot(nlsresun)
  nlstools::test.nlsResiduals(nlsresun)  
}
# ___ ( ModelDiagnostics_Aqueous) ____________________


# ///////////////////////////////////////////////////
# === [ ModelDiagnostics_Aqueous_BoxCox] ====================
# --- package nlstools
 ModelDiagnostics_Aqueous_BoxCox <- function(modelfit = BCF.ModelFit.BoxCox) {
  nlsresbc <- nlstools::nlsResiduals(modelfit)
  plot(nlsresbc)
  nlstools::test.nlsResiduals(nlsresbc)  
}
# ___ ( ModelDiagnostics_Aqueous_BoxCox) ____________________


# ///////////////////////////////////////////////
# === [ ModelDiagnostics_Aqueous_Ln] ====================
# --- package nlstools
 ModelDiagnostics_Aqueous_Ln <- function(modelfit = BCF.ModelFit.Ln) {
  nlsresln <- nlstools::nlsResiduals(modelfit)
  plot(nlsresln)
  nlstools::test.nlsResiduals(nlsresln)  
}
# ___ ( ModelDiagnostics_Aqueous_Ln) ____________________



# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@


# ///////////////////////////////////////////
# === [FitModel_Aqueous] ====================
# --- FINAL bcmfR
# --- dataframe: dataframe with "time.data" & "cfish.data"
# --- testparam: vector with "cwater" & "tdepur"
FitModel_Aqueous <- function(datacfish = DATA.Aqueous.Work, 
                             testparam = TEST.Aqueous.Design,
                             numcontrol= NUM.Control.nls,
                             lnstarter = 1e-9,
                             startfit  = NULL,
                             trace     = FALSE,
                             savefit   = TRUE) {
  
  if (is.null(startfit)) {
    # returns fitk1, fitk2 vector
    startfit <- StartFit_Aqueous(datacfish = datacfish,
                                 testparam = testparam,
                                 lnstarter = lnstarter)
    
  } else {
    # if names lack, add them
    if (is.null(names(startfit)))
      names(startfit) <- c("fitk1", "fitk2")
  }
  
  if (savefit)
    assign("BCF.StartFit", startfit, envir = .GlobalEnv)
  
  #-- retrieve scalar param values for simulation, only for Aqueous
  cwater <- testparam["cwater", "value"]
  tdepur <- testparam["tdepur", "value"]
  
  modfitcfishjoint <- nls(cfish.data ~
                            RunModel_Aqueous(time.data, cwater, tdepur,
                                                fitk1, fitk2),
                          data    = datacfish,
                          start   = startfit,
                          control = numcontrol,
                          trace   = trace)
  
  if (savefit)
    assign("BCF.ModelFit", modfitcfishjoint, envir = .GlobalEnv)
  
  coefsmodfit <- coef(modfitcfishjoint)
  names(coefsmodfit) <- c("k1","k2")
  
  if (savefit)
    assign("BCF.ParamFit", coefsmodfit , envir = .GlobalEnv)
  
  return(summary(modfitcfishjoint))
  
}
# ___ (FitModel_Aqueous) ____________________


# //////////////////////////////////////////////////
# === [FitModel_Aqueous_BoxCox] ====================
# --- FINAL bcmfR
FitModel_Aqueous_BoxCox <- function(datacfish = DATA.Aqueous.Work, 
                                    testparam = TEST.Aqueous.Design,
                                    lambda    = BCF.LambdaEst["fit"],
                                    numcontrol= NUM.Control.nls,
                                    lnstarter = 1e-9,
                                    startfit  = NULL,
                                    trace     = FALSE,
                                    savefit   = TRUE) {
  #..........
  
  if (is.null(startfit)) {
    # returns fitk1, fitk2 vector
    startfit <- StartFit_Aqueous(datacfish = datacfish,
                                 testparam = testparam,
                                 lnstarter = lnstarter)
    
  } else {
    # if names lack, add them
    if (is.null(names(startfit)))
      names(startfit) <- c("fitk1", "fitk2")
  }
  
  if (savefit)
    assign("BCF.StartFit", startfit, envir = .GlobalEnv)
  
  # ..........
  
  # -- precalculate boxcox-transform in DATA.Aqueous.Work
  datacfish <- dataTrans_BoxCox(datacfish,
                                lambda = lambda,
                                lnstarter = lnstarter)
  
  # -- adds a column bc.cfish.data to the work sheet
  if (savefit)
    assign("DATA.Aqueous.Work", datacfish, envir = .GlobalEnv)
  
  # ..........
  
  # -- retrieve scalar param values for simulation, only for Aqueous
  cwater <- testparam["cwater", "value"]
  tdepur <- testparam["tdepur", "value"]
  
  # ..........
  
  # -- fit nonlinear model with BoxCox transformation parameter lambda
  modfitcfishjoint <- nls(bc.cfish.data ~
                            car::bcPower(RunModel_Aqueous(time.data, cwater, tdepur,
                                      fitk1, fitk2) + lnstarter, lambda),
                          data    = datacfish,
                          start   = startfit,
                          control = numcontrol,
                          trace   = trace)
  
  if (savefit)
    assign("BCF.ModelFit.BoxCox", modfitcfishjoint, envir = .GlobalEnv)
  
  # ..........
  
  # -- rename the coef vector for paramFit
  coefvecbc <- coef(modfitcfishjoint)
  names(coefvecbc) <- c("k1","k2")
  
  if (savefit) {
    assign("BCF.ParamFit.BoxCox", coefvecbc, envir = .GlobalEnv)
    assign("BCF.ParamFit.Lambda", lambda, envir = .GlobalEnv)
  }
  
  return(summary(modfitcfishjoint))
  
}
# ___ (FitModel_Aqueous_BoxCox) ____________________


# //////////////////////////////////////////////
# === [FitModel_Aqueous_Ln] ====================
# --- FINAL bcmfR
FitModel_Aqueous_Ln <- function(datacfish = DATA.Aqueous.Work, 
                                testparam = TEST.Aqueous.Design,
                                numcontrol= NUM.Control.nls,
                                lnstarter = 1e-9,
                                startfit  = NULL, # use vector
                                trace     = FALSE,
                                savefit   = TRUE) {
  
  if (is.null(startfit)) {
    
    # returns fitk1, fitk2 vector
    startfit <- StartFit_Aqueous(datacfish = datacfish,
                                 testparam = testparam,
                                 lnstarter = lnstarter)
    
  } else {
    if (is.null(names(startfit)))
      names(startfit) <- c("fitk1", "fitk2")
  }
  
  if (savefit)
    assign("BCF.StartFit", startfit, envir = .GlobalEnv)
  
  #..........
  
  #-- retrieve scalar param values for simulation, only for Aqueous
  cwater <- testparam["cwater", "value"]
  tdepur <- testparam["tdepur", "value"]
  
  #..........
  
  #-- precalculate ln-transform in DATA.Aqueous.Work
  datacfish <- dataTrans_Ln(datacfish, lnstarter = lnstarter)
  
  #-- save transform in work sheet (ln.cfish.data)
  if (savefit)
    assign("DATA.Aqueous.Work", datacfish, envir = .GlobalEnv)

  #..........
  
  #-- fit the model both sides Ln-transformed  TRY????
  modfitcfishjoint <- nls(ln.cfish.data ~
                            log(RunModel_Aqueous(time.data, cwater, tdepur,
                                                 fitk1, fitk2) + lnstarter),
                          data    = datacfish,
                          start   = startfit,
                          control = numcontrol,
                          trace   = trace)
  #..........
  
  if (savefit)
    assign("BCF.ModelFit.Ln", modfitcfishjoint, envir = .GlobalEnv)
  
  #-- rename the coefs
  coefvecln <- coef(modfitcfishjoint)
  names(coefvecln) <- c("k1","k2")
  
  #-- store the parameter fit
  if (savefit)
    # only modelFit does this
    assign("BCF.ParamFit.Ln", coefvecln, envir = .GlobalEnv)

  return(summary(modfitcfishjoint))

}
# ___ (FitModel_Aqueous_Ln) _____________________



# >>>>>>>>>>>>>>>>>>>>>>  ONAF!!
# === [modelPredBest_Aqueous] ====================
modelPredBest_Aqueous <- function(datawork = DATA.Aqueous.Work,
                                  paramfit = BCF.ParamFit,
                                  testparam = TEST.Aqueous.Design,
                                  savefit = TRUE,
                                  steps   = 60,
                                  tmin    = NULL,
                                  tmax    = NULL,
                                  textra  = NULL) {
  
  timesumm <- dataTime_Summary(datawork)
  
  if (is.null(tmin)) {
    tmin <- timesumm["min"]
  }
  
  if (is.null(tmax)) {
    tmax <- timesumm["max"]
  }
  
  if (!is.null(textra)) {
    trange <- tmax - tmin
    textend <- textra*trange
    
    tmin <- max(0., tmin - textend)
    tmax <- tmax + textend
  }
  
  timepred <- seq(tmin, tmax, length.out = steps + 1)
  
  cfishpred <- RunModel_Aqueous(timepred, paramfit["k1"], paramfit["k2"])
  
  predframe <- data.frame(best.time = timepred, best.cfish = cfishpred)
  
  if (savefit)
    assign("BCF.PredBest", predframe, envir = .GlobalEnv)
  
  return(predframe)
}
# ___ (modelPredBest_Aqueous) ____________________



# === [RunModel_Aqueous] ====================
# --- FINAL bcmfR
# --- REMOVED arg testparam!!!!
# --- uses C0,d as parameter, instead of testparam vector
RunModel_Aqueous <- function(time, cwater, tdepur, park1, park2) {
  
  term1  <- cwater*park1/park2
  term2  <- term1*(exp(park2*tdepur) - 1)
  
  cfish <- ifelse(time < tdepur,
                  term1*(1 - exp(-park2*time)),
                  term2*exp(-park2*time))
  
  return(cfish)
}
# ___ (RunModel_Aqueous) ____________________



# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

# === [ModelTrans_BoxCox_Aqueous] ====================
ModelTrans_BoxCox_Aqueous <- function() {
  
  if (exists("BCF.ModelFit"))
    modelfit <- BCF.ModelFit
  else
    modelfit <- FitModel_Aqueous()
  
  lambdaest <- ModelTrans_BoxCox(modelfit)   # zit in base.R
  
  assign("BCF.LambdaEst", lambdaest, envir = .GlobalEnv)
  
  return(lambdaest)
  
}
# ___ (ModelTrans_BoxCox_Aqueous) ____________________



# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@


# //////////////////////////////////////////////
# === [PlotBestFit_Aqueous] ====================
# ~~~ assumes BCF.BestFit to exist, saved by bestFit_Aqueous()
PlotBestFit_Aqueous <- function(testdata    = DATA.Aqueous.Work,
                                testmeas    = TEST.Aqueous.Measured,
                                bestfitdata = BCF.BestFit,
                                plotstyle   = PLOT.StyleFit,
                                plottheme   = PLOT.Theme,
                                plottitle   = PLOT.Title.BestFit,
                                plotxlab    = NULL,
                                plotylab    = NULL) {

  if (is.null(plotxlab)) {
    # -- x label text
    plotxlab <- paste(testmeas["time","name"], "  (",
                      testmeas["time","unit"], ")", sep = "")
  }
  
  if (is.null(plotylab)) {
    # -- y label text
    plotylab <- paste(testmeas["cfish","name"], "  (",
                      testmeas["cfish","unit"], ")", sep = "")
  }
  
  #-- scatter of measured test data
  plot0 <- ggplot2::ggplot(testdata, ggplot2::aes(time.data, cfish.data))
  
  # -- add scatter and bestfit curve
  plot1 <- plot0 +
    
      ggplot2::geom_line(data = bestfitdata,
                ggplot2::aes(time.best, cfish.best),
                
                size = plotstyle$linebest$size,
                linetype = plotstyle$linebest$style,
                color = plotstyle$linebest$color) +
      
      ggplot2::geom_point(shape = plotstyle$symbol$shape,
                 size  = plotstyle$symbol$size,
                 color = plotstyle$symbol$color) +
    
      ggplot2::ggtitle(plottitle) +
    
      ggplot2::labs(x = plotxlab, y = plotylab)
  
  # -- add theme 
  plot2 <- plot1 + expand_theme(plottheme)
  
  print(plot2)
  
}
# ___ (PlotBestFit_Aqueous) ____________________


#//////////////////////////////////////////////////////
# === [PlotBestFit_Aqueous_BoxCox] ====================
# ~~~ assumes BCF.BestFit.BoxCox to exist, saved by bestFit_Aqueous()
# ~~~ assumes bc.cfish.data in DATA.Aqueous.Work
PlotBestFit_Aqueous_BoxCox <- function(testdata    = DATA.Aqueous.Work,
                                       testmeas    = TEST.Aqueous.Measured,
                                       bestfitdata = BCF.BestFit.BoxCox,
                                       plotstyle   = PLOT.StyleFit.BoxCox,
                                       plottheme   = PLOT.Theme,
                                       plottitle   = PLOT.Title.BestFit.BoxCox,
                                       plotxlab    = NULL,
                                       plotylab    = NULL) {

  if (is.null(plotxlab)) {
    # -- x label text
    plotxlab <- paste(testmeas["time","name"], "  (",
                      testmeas["time","unit"], ")", sep = "")
  }
  
  if (is.null(plotylab)) {
    # -- y label text
    plotylab <- paste("Box-Cox ", testmeas["cfish","name"], "  (",
                      testmeas["cfish","unit"], ")", sep = "")
  }
  
  #-- scatter of measured test data, boxcox-transformed
  plot0 <- ggplot2::ggplot(testdata, ggplot2::aes(time.data, bc.cfish.data))
  
  # -- add scatter and bestfit curve
  plot1 <- plot0 +
    
      ggplot2::geom_line(data = bestfitdata,
                ggplot2::aes(time.best, bc.cfish.best),
                
                size = plotstyle$linebest$size,
                linetype = plotstyle$linebest$style,
                color = plotstyle$linebest$color) +
    
      ggplot2::geom_point(shape = plotstyle$symbol$shape,
                 size  = plotstyle$symbol$size,
                 color = plotstyle$symbol$color) +
    
      ggplot2::ggtitle(plottitle) +
    
      ggplot2::labs(x = plotxlab, y = plotylab)
  
  # -- add theme 
  plot2 <- plot1 + expand_theme(plottheme)
  
  print(plot2)
  
}
# ___ (PlotBestFit_Aqueous_BoxCox) ____________________


# /////////////////////////////////////////////////
# === [PlotBestFit_Aqueous_Ln] ====================
# ~~~ assumes BCF.BestFit.Ln to exist, saved by bestFit_Aqueous()
# ~~~ assumes ln.cfish.data in DATA.Aqueous.Work
PlotBestFit_Aqueous_Ln <- function(testdata = DATA.Aqueous.Work,
                                   testmeas     = TEST.Aqueous.Measured,
                                   bestfitdata  = BCF.BestFit.Ln,
                                   plotstyle   = PLOT.StyleFit.Ln,
                                   plottheme   = PLOT.Theme,
                                   plottitle   = PLOT.Title.BestFit.Ln,
                                   plotxlab    = NULL,
                                   plotylab    = NULL) {

  if (is.null(plotxlab)) {
    # -- x label text
    plotxlab <- paste(testmeas["time","name"], "  (",
                      testmeas["time","unit"], ")", sep = "")
  }
  
  if (is.null(plotylab)) {
    # -- y label text
    plotylab <- paste("Ln ", testmeas["cfish","name"], "  (",
                      testmeas["cfish","unit"], ")", sep = "")
  }
  
  #-- scatter of measured test data, ln-transformed
  plot0 <- ggplot2::ggplot(testdata, ggplot2::aes(time.data, ln.cfish.data))
  
  # -- add scatter and bestfit curve
  plot1 <- plot0 +
    
      ggplot2::geom_line(data = bestfitdata, 
                ggplot2::aes(time.best, ln.cfish.best),
                
                size = plotstyle$linebest$size,
                linetype = plotstyle$linebest$style,
                color = plotstyle$linebest$color) +
    
      ggplot2::geom_point(shape = plotstyle$symbol$shape,
                 size  = plotstyle$symbol$size,
                 color = plotstyle$symbol$color) +
    
      ggplot2::ggtitle(plottitle) +
    
      ggplot2::labs(x = plotxlab, y = plotylab)
  
  # -- add theme 
  plot2 <- plot1 + expand_theme(plottheme)
  
  print(plot2)
  
}
# ___ (PlotBestFit_Aqueous_Ln) ____________________



# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@


# //////////////////////////////////////////////
# === [PlotConfFit_Aqueous] ====================
# ~~~ assumes BCF.BestFit to exist, saved by RunBestFit_Aqueous()
PlotConfFit_Aqueous <- function(testdata    = DATA.Aqueous.Work,
                                testmeas    = TEST.Aqueous.Measured,
                                bestfitdata = BCF.BestFit,
                                conffitdata = BCF.ConfFit,
                                plotstyle   = PLOT.StyleFit,
                                plottheme   = PLOT.Theme,
                                plottitle   = PLOT.Title.ConfFit,
                                plotxlab    = NULL,
                                plotylab    = NULL) {

  if (is.null(plotxlab)) {
    # -- x label text
    plotxlab <- paste(testmeas["time","name"], "  (",
                      testmeas["time","unit"], ")", sep = "")
  }
  
  if (is.null(plotylab)) {
    # -- y label text
    plotylab <- paste(testmeas["cfish","name"], "  (",
                      testmeas["cfish","unit"], ")", sep = "")
  }
  
  #-- scatter of measured test data
  plot0 <- ggplot2::ggplot(testdata, ggplot2::aes(time.data, cfish.data))
  
  # -- add scatter and bestfit curve
  plot1 <- plot0 +
    
      ggplot2::geom_line(data = bestfitdata,
                ggplot2::aes(time.best, cfish.best),
                
                size = plotstyle$linebest$size,
                linetype = plotstyle$linebest$style,
                color = plotstyle$linebest$color) +
    
      ggplot2::geom_line(data = conffitdata,
                ggplot2::aes(time.conf, cfish.conflo),
                
                size = plotstyle$lineconf$size,
                linetype = plotstyle$lineconf$style,
                color = plotstyle$lineconf$color) +
    
      ggplot2::geom_line(data = conffitdata,
                ggplot2::aes(time.conf, cfish.confup),
                
                size = plotstyle$lineconf$size,
                linetype = plotstyle$lineconf$style,
                color = plotstyle$lineconf$color) +
    
      ggplot2::geom_point(shape = plotstyle$symbol$shape,
                 size  = plotstyle$symbol$size,
                 color = plotstyle$symbol$color) +
    
      ggplot2::ggtitle(plottitle) +
    
      ggplot2::labs(x = plotxlab, y = plotylab)
  
  # -- add theme 
  plot2 <- plot1 + expand_theme(plottheme)
  
  print(plot2)
  
}
# ___ (PlotConfFit_Aqueous) ____________________


# /////////////////////////////////////////////////////
# === [PlotConfFit_Aqueous_BoxCox] ====================
# ~~~ assumes BMF.BestFit to exist, saved by RunBestFit_Aqueous()
PlotConfFit_Aqueous_BoxCox <- function(testdata    = DATA.Aqueous.Work,
                                       testmeas    = TEST.Aqueous.Measured,
                                       bestfitdata = BCF.BestFit.BoxCox,
                                       conffitdata = BCF.ConfFit.BoxCox,
                                       plotstyle   = PLOT.StyleFit.BoxCox,
                                       plottheme   = PLOT.Theme,
                                       plottitle   = PLOT.Title.ConfFit.BoxCox,
                                       plotxlab    = NULL,
                                       plotylab    = NULL) {

  if (is.null(plotxlab)) {
    # -- x label text
    plotxlab <- paste(testmeas["time","name"], "  (",
                      testmeas["time","unit"], ")", sep = "")
  }
  
  if (is.null(plotylab)) {
    # -- y label text
    plotylab <- paste("BoxCox ", testmeas["cfish","name"], "  (",
                      testmeas["cfish","unit"], ")", sep = "")
  }
  
  #-- scatter of measured test data
  plot0 <- ggplot2::ggplot(testdata, ggplot2::aes(time.data, bc.cfish.data))
  
  # -- add scatter, bestfit curve, and conflo & confup curves
  plot1 <- plot0 +
    
      ggplot2::geom_line(data = bestfitdata, 
                ggplot2::aes(time.best, bc.cfish.best),
                
                size = plotstyle$linebest$size,
                linetype = plotstyle$linebest$style,
                color = plotstyle$linebest$color) +
    
      ggplot2::geom_line(data = conffitdata, 
                ggplot2::aes(time.conf, bc.cfish.conflo),
                
                size = plotstyle$lineconf$size,
                linetype = plotstyle$lineconf$style,
                color = plotstyle$lineconf$color) +
    
      ggplot2::geom_line(data = conffitdata, 
                ggplot2::aes(time.conf, bc.cfish.confup),
                
                size = plotstyle$lineconf$size,
                linetype = plotstyle$lineconf$style,
                color = plotstyle$lineconf$color) +
    
      ggplot2::geom_point(shape = plotstyle$symbol$shape,
                 size  = plotstyle$symbol$size,
                 color = plotstyle$symbol$color) +
    
      ggplot2::ggtitle(plottitle) +
    
      ggplot2::labs(x = plotxlab, y = plotylab)
  
  # -- add theme 
  plot2 <- plot1 + expand_theme(plottheme)
  
  print(plot2)
  
  
}
# ___ (PlotConfFit_Aqueous_BoxCox) ____________________


# /////////////////////////////////////////////////
# === [PlotConfFit_Aqueous_Ln] ====================
# ~~~ assumes BMF.BestFit to exist, saved by RunBestFit_Aqueous()
PlotConfFit_Aqueous_Ln <- function(testdata    = DATA.Aqueous.Work,
                                   testmeas    = TEST.Aqueous.Measured,
                                   bestfitdata = BCF.BestFit.Ln,
                                   conffitdata = BCF.ConfFit.Ln,
                                   plotstyle   = PLOT.StyleFit.Ln,
                                   plottheme   = PLOT.Theme,
                                   plottitle   = PLOT.Title.ConfFit.Ln,
                                   plotxlab    = NULL,
                                   plotylab    = NULL) {
  
  if (is.null(plotxlab)) {
    # -- x label text
    plotxlab <- paste(testmeas["time","name"], "  (",
                      testmeas["time","unit"], ")", sep = "")
  }
  
  if (is.null(plotylab)) {
    # -- y label text
    plotylab <- paste("Ln ", testmeas["cfish","name"], "  (",
                      testmeas["cfish","unit"], ")", sep = "")
  }
  
  #-- scatter of measured test data
  plot0 <- ggplot2::ggplot(testdata, ggplot2::aes(time.data, ln.cfish.data))
  
  # -- add scatter, bestfit curve, and conflo & confup curves
  plot1 <- plot0 +
    
      ggplot2::geom_line(data = bestfitdata,
                ggplot2::aes(time.best, ln.cfish.best),
                
                size = plotstyle$linebest$size,
                linetype = plotstyle$linebest$style,
                color = plotstyle$linebest$color) +
    
      ggplot2::geom_line(data = conffitdata,
                ggplot2::aes(time.conf, ln.cfish.conflo),
                
                size = plotstyle$lineconf$size,
                linetype = plotstyle$lineconf$style,
                color = plotstyle$lineconf$color) +
    
      ggplot2::geom_line(data = conffitdata,
                ggplot2::aes(time.conf, ln.cfish.confup),
                
                size = plotstyle$lineconf$size,
                linetype = plotstyle$lineconf$style,
                color = plotstyle$lineconf$color) +
    
    ggplot2::geom_point(shape = plotstyle$symbol$shape,
               size  = plotstyle$symbol$size,
               color = plotstyle$symbol$color) +
    
    ggplot2::ggtitle(plottitle) +
    
    ggplot2::labs(x = plotxlab, y = plotylab)
  
  # -- add theme 
  plot2 <- plot1 + expand_theme(plottheme)
  
  print(plot2)
  
}
# ___ (PlotConfFit_Aqueous_Ln) ____________________



# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

# ////////////////////////////////////////////////////
# === [PlotInvBestFit_Aqueous_BoxCox] ====================
# ~~~ assumes BCF.BestFit.Ln to exist, saved by bestFit_Aqueous()
# ~~~ assumes ln.cfish.data in DATA.Aqueous.Work
PlotInvBestFit_Aqueous_BoxCox <- function(testdata    = DATA.Aqueous.Work,
                                          testmeas    = TEST.Aqueous.Measured,
                                          bestfitdata = BCF.BestFit.BoxCox,
                                          plotstyle   = PLOT.StyleFit.BoxCox,
                                          plottheme   = PLOT.Theme,
                                          plottitle   = PLOT.Title.BestFit.BoxCox,
                                          plotxlab    = NULL,
                                          plotylab    = NULL) {

  if (is.null(plotxlab)) {
    # -- x label text
    plotxlab <- paste(testmeas["time","name"], "  (",
                      testmeas["time","unit"], ")", sep = "")
  }
  
  if (is.null(plotylab)) {
    # -- y label text
    plotylab <- paste(testmeas["cfish","name"], "  (",
                      testmeas["cfish","unit"], ")", sep = "")
  }
  
  # -- scatter of measured test data, UN-transformed
  plot0 <- ggplot2::ggplot(testdata, ggplot2::aes(time.data, cfish.data))
  
  # -- add scatter and bestfit curve, back-transformed
  plot1 <- plot0 +
    
      ggplot2::geom_line(data = bestfitdata,
                ggplot2::aes(time.best, inv.bc.cfish.best),
                
                size = plotstyle$linebest$size,
                linetype = plotstyle$linebest$style,
                color = plotstyle$linebest$color) +
    
      ggplot2::geom_point(shape = plotstyle$symbol$shape,
                 size  = plotstyle$symbol$size,
                 color = plotstyle$symbol$color) +
    
      ggplot2::ggtitle(plottitle) +
    
      ggplot2::labs(x = plotxlab, y = plotylab)
  
  # -- add theme 
  plot2 <- plot1 + expand_theme(plottheme)
  
  print(plot2)
  
}
# ___ (PlotInvBestFit_Aqueous_BoxCox) ____________________


# ////////////////////////////////////////////////////
# === [PlotInvBestFit_Aqueous_Ln] ====================
# ~~~ assumes BCF.BestFit.Ln to exist, saved by bestFit_Aqueous()
# ~~~ assumes ln.cfish.data in DATA.Aqueous.Work
PlotInvBestFit_Aqueous_Ln <- function(testdata    = DATA.Aqueous.Work,
                                      testmeas    = TEST.Aqueous.Measured,
                                      bestfitdata = BCF.BestFit.Ln,
                                      plotstyle   = PLOT.StyleFit.Ln,
                                      plottheme   = PLOT.Theme,
                                      plottitle   = PLOT.Title.BestFit.Ln,
                                      plotxlab    = NULL,
                                      plotylab    = NULL) {

  if (is.null(plotxlab)) {
    # -- x label text
    plotxlab <- paste(testmeas["time","name"], "  (",
                      testmeas["time","unit"], ")", sep = "")
  }
  
  if (is.null(plotylab)) {
    # -- y label text
    plotylab <- paste(testmeas["cfish","name"], "  (",
                      testmeas["cfish","unit"], ")", sep = "")
  }
  
  # -- scatter of measured test data, UN-transformed
  plot0 <- ggplot2::ggplot(testdata, ggplot2::aes(time.data, cfish.data))
  
  # -- add scatter and bestfit curve, back-transformed
  plot1 <- plot0 +
    
      ggplot2::geom_line(data = bestfitdata,
                ggplot2::aes(time.best, inv.ln.cfish.best),
                
                size = plotstyle$linebest$size,
                linetype = plotstyle$linebest$style,
                color = plotstyle$linebest$color) +
    
      ggplot2::geom_point(shape = plotstyle$symbol$shape,
                 size  = plotstyle$symbol$size,
                 color = plotstyle$symbol$color) +
    
      ggplot2::ggtitle(plottitle) +
    
      ggplot2::labs(x = plotxlab, y = plotylab)
  
  # -- add theme 
  plot2 <- plot1 + expand_theme(plottheme)
  
  print(plot2)
  
}
# ___ (PlotInvBestFit_Aqueous_Ln) ____________________



# ////////////////////////////////////////////////////
# === [PlotInvConfFit_Aqueous_BoxCox] ====================
# ~~~ assumes BCF.BestFit.Ln to exist, saved by RunBestFit_Aqueous()
# ~~~ assumes ln.cfish.data in DATA.Aqueous.Work
PlotInvConfFit_Aqueous_BoxCox <- function(testdata    = DATA.Aqueous.Work,
                                          testmeas    = TEST.Aqueous.Measured,
                                          bestfitdata = BCF.BestFit.BoxCox,
                                          conffitdata = BCF.ConfFit.BoxCox,
                                          plotstyle   = PLOT.StyleFit.BoxCox,
                                          plottheme   = PLOT.Theme,
                                          plottitle   = PLOT.Title.ConfFit.BoxCox,
                                          plotxlab    = NULL,
                                          plotylab    = NULL) {

  if (is.null(plotxlab)) {
    # -- x label text
    plotxlab <- paste(testmeas["time","name"], "  (",
                      testmeas["time","unit"], ")", sep = "")
  }
  
  if (is.null(plotylab)) {
    # -- y label text
    plotylab <- paste(testmeas["cfish","name"], "  (",
                      testmeas["cfish","unit"], ")", sep = "")
  }
  
  # -- scatter of measured test data, UN-transformed
  plot0 <- ggplot2::ggplot(testdata, ggplot2::aes(time.data, cfish.data))
  
  # -- add scatter and bestfit curve, back-transformed
  plot1 <- plot0 +
    
      ggplot2::geom_line(data = bestfitdata,
                ggplot2::aes(time.best, inv.bc.cfish.best),
                
                size = plotstyle$linebest$size,
                linetype = plotstyle$linebest$style,
                color = plotstyle$linebest$color) +
    
      ggplot2::geom_line(data = conffitdata,
                ggplot2::aes(time.conf, inv.bc.cfish.conflo),
                
                size = plotstyle$lineconf$size,
                linetype = plotstyle$lineconf$style,
                color = plotstyle$lineconf$color) +
    
      ggplot2::geom_line(data = conffitdata,
                ggplot2::aes(time.conf, inv.bc.cfish.confup),
                
                size = plotstyle$lineconf$size,
                linetype = plotstyle$lineconf$style,
                color = plotstyle$lineconf$color) +
    
      ggplot2::geom_point(shape = plotstyle$symbol$shape,
                 size  = plotstyle$symbol$size,
                 color = plotstyle$symbol$color) +
    
      ggplot2::ggtitle(plottitle) +
    
      ggplot2::labs(x = plotxlab, y = plotylab)
  
  # -- add theme 
  plot2 <- plot1 + expand_theme(plottheme)
  
  print(plot2)
  
}
# ___ (PlotInvConfFit_Aqueous_BoxCox) ____________________



# ////////////////////////////////////////////////////
# === [PlotInvConfFit_Aqueous_Ln] ====================
# ~~~ assumes BCF.BestFit.Ln to exist, saved by RunBestFit_Aqueous()
# ~~~ assumes ln.cfish.data in DATA.Aqueous.Work
PlotInvConfFit_Aqueous_Ln <- function(testdata    = DATA.Aqueous.Work,
                                      testmeas    = TEST.Aqueous.Measured,
                                      bestfitdata = BCF.BestFit.Ln,
                                      conffitdata = BCF.ConfFit.Ln,
                                      plotstyle   = PLOT.StyleFit.Ln,
                                      plottheme   = PLOT.Theme,
                                      plottitle   = PLOT.Title.ConfFit.Ln,
                                      plotxlab    = NULL,
                                      plotylab    = NULL) {

  if (is.null(plotxlab)) {
    # -- x label text
    plotxlab <- paste(testmeas["time","name"], "  (",
                      testmeas["time","unit"], ")", sep = "")
  }
  
  if (is.null(plotylab)) {
    # -- y label text
    plotylab <- paste(testmeas["cfish","name"], "  (",
                      testmeas["cfish","unit"], ")", sep = "")
  }
  
  # -- scatter of measured test data, UN-transformed
  plot0 <- ggplot2::ggplot(testdata, ggplot2::aes(time.data, cfish.data))
  
  # -- add scatter and bestfit curve, back-transformed
  plot1 <- plot0 +
    
      ggplot2::geom_line(data = bestfitdata, 
                ggplot2::aes(time.best, inv.ln.cfish.best),
                
                size = plotstyle$linebest$size,
                linetype = plotstyle$linebest$style,
                color = plotstyle$linebest$color) +
    
      ggplot2::geom_line(data = conffitdata, 
                ggplot2::aes(time.conf, inv.ln.cfish.conflo),
                
                size = plotstyle$lineconf$size,
                linetype = plotstyle$lineconf$style,
                color = plotstyle$lineconf$color) +
    
      ggplot2::geom_line(data = conffitdata, 
                ggplot2::aes(time.conf, inv.ln.cfish.confup),
                
                size = plotstyle$lineconf$size,
                linetype = plotstyle$lineconf$style,
                color = plotstyle$lineconf$color) +
    
      ggplot2::geom_point(shape = plotstyle$symbol$shape,
                 size  = plotstyle$symbol$size,
                 color = plotstyle$symbol$color) +
    
      ggplot2::ggtitle(plottitle) +
     
      ggplot2::labs(x = plotxlab, y = plotylab)
  
  # -- add theme 
  plot2 <- plot1 + expand_theme(plottheme)
  
  print(plot2)
  
}
# ___ (PlotInvBestFit_Aqueous_Ln) ____________________



# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

# //////////////////////////////////////////
# === [RunBestFit_Aqueous] ====================
# --- direct implementation
RunBestFit_Aqueous <- function(datawork  = DATA.Aqueous.Work,
                               testparam = TEST.Aqueous.Design,
                               paramfit  = BCF.ParamFit,
                               savefit   = TRUE,
                               steps     = 60,
                               tmin      = NULL,
                               tmax      = NULL,
                               textra    = NULL) {
  
  cwater <- testparam["cwater", "value"]
  tdepur <- testparam["tdepur", "value"]
  
  timesumm <- dataTime_Summary(datawork)
  
  if (is.null(tmin)) {
    tmin <- timesumm["min"]
  }
  
  if (is.null(tmax)) {
    tmax <- timesumm["max"]
  }
  
  if (!is.null(textra)) {
    trange <- tmax - tmin
    textend <- textra*trange
    
    tmin <- max(1e-9, tmin - textend)
    tmax <- tmax + textend
  }
  
  # -- time sequence supplemented with exact tdepur
  timepred <- sort(c(seq(tmin, tmax, length.out = steps + 1), tdepur))
  
  # -- run the model
  cfishpred <- RunModel_Aqueous(timepred, cwater, tdepur,
                                paramfit["k1"], paramfit["k2"])
  
  # -- put into dataframe
  predframe <- data.frame(time.best = timepred, cfish.best = cfishpred)
  
  if (savefit)
    assign("BCF.BestFit", predframe, envir = .GlobalEnv)
  
  top    <- head(predframe, n=10)
  bottom <- tail(predframe, n=10)
  
  print(top)
  cat("   ...")
  cat("\n")
  print(bottom)
  
  #return(predframe)
}
# ___ (RunBestFit_Aqueous) ____________________


# /////////////////////////////////////////////////
# === [RunBestFit_Aqueous_BoxCox] ====================
# --- direct implementation
RunBestFit_Aqueous_BoxCox <- function(datawork = DATA.Aqueous.Work,
                                      testparam = TEST.Aqueous.Design,
                                      parfitbc = BCF.ParamFit.BoxCox,
                                      lambda   = BCF.ParamFit.Lambda,
                                      lnstarter = 1e-9,
                                      savefit = TRUE,
                                      steps   = 60,
                                      tmin    = NULL,
                                      tmax    = NULL,
                                      textra  = NULL) {
  
  cwater <- testparam["cwater", "value"]
  tdepur <- testparam["tdepur", "value"]
  
  timesumm <- dataTime_Summary(datawork)
  
  if (is.null(tmin)) {
    tmin <- timesumm["min"]
  }
  
  if (is.null(tmax)) {
    tmax <- timesumm["max"]
  }
  
  if (!is.null(textra)) {
    trange <- tmax - tmin
    textend <- textra*trange
    
    tmin <- max(1e-9, tmin - textend)
    tmax <- tmax + textend
  }
  
  # -- time sequence supplemented with exact tdepur
  timepred <- sort(c(seq(tmin, tmax, length.out = steps + 1), tdepur))
  
  # -- untransformed modelrun on bc-transformed fitted params
  invbccfishpred <- RunModel_Aqueous(timepred, cwater, tdepur,
                                     parfitbc["k1"], parfitbc["k2"])
  
  # -- the bcpow-transformed prediction is just that
  bccfishpred <- car::bcPower(invbccfishpred + lnstarter, lambda)
  
  # -- for PlotInvBestFit we store both
  predframe <- data.frame(time.best = timepred,
                          bc.cfish.best = bccfishpred,
                          inv.bc.cfish.best = invbccfishpred)
  
  if (savefit)
    assign("BCF.BestFit.BoxCox", predframe, envir = .GlobalEnv)
  
  top    <- head(predframe, n=10)
  bottom <- tail(predframe, n=10)
  
  print(top)
  cat("   ...")
  cat("\n")
  print(bottom)
  
  # return(predframe)
}
# ___ (RunBestFit_Aqueous_BoxCox) ____________________


# /////////////////////////////////////////////
# === [RunBestFit_Aqueous_Ln] ====================
# --- direct implementation
RunBestFit_Aqueous_Ln <- function(datawork = DATA.Aqueous.Work,
                                  testparam = TEST.Aqueous.Design,
                                  parfitln  = BCF.ParamFit.Ln,
                                  lnstarter = 1e-9,
                                  savefit = TRUE,
                                  steps   = 60,
                                  tmin    = NULL,
                                  tmax    = NULL,
                                  textra  = NULL) {
  
  cwater <- testparam["cwater", "value"]
  tdepur <- testparam["tdepur", "value"]
  
  timesumm <- dataTime_Summary(datawork)
  
  # --- clipping
  # cfishsumm <- dataCFish_Summary(datawork)
  # lncfmin <- log(cfishsumm["min"])
  
  if (is.null(tmin)) {
    tmin <- timesumm["min"]
  }
  
  if (is.null(tmax)) {
    tmax <- timesumm["max"]
  }
  
  if (!is.null(textra)) {
    trange <- tmax - tmin
    textend <- textra*trange
    
    tmin <- max(1e-9, tmin - textend)
    tmax <- tmax + textend
  }
  
  # -- time sequence supplemented with exact tdepur
  timepred <- sort(c(seq(tmin, tmax, length.out = steps + 1), tdepur))
  
  # -- untransformed modelrun on ln-transformed fitted params
  invlncfishpred <- RunModel_Aqueous(timepred, cwater, tdepur,
                                     parfitln["k1"], parfitln["k2"])
  
  # -- the ln-transformed prediction is just the log
  lncfishpred <- log(invlncfishpred + lnstarter)
  
  # -- for PlotInvBestFit we store both
  predframe <- data.frame(time.best = timepred,
                          ln.cfish.best = lncfishpred,
                          inv.ln.cfish.best = invlncfishpred)
  
  if (savefit)
    assign("BCF.BestFit.Ln", predframe, envir = .GlobalEnv)
  
  top    <- head(predframe, n=10)
  bottom <- tail(predframe, n=10)
  
  print(top)
  cat("   ...")
  cat("\n")
  print(bottom)
  
  # return(predframe)
}
# ___ (RunBestFit_Aqueous_Ln) ____________________



# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

# /////////////////////////////////////////////
# === [RunConfFit_Aqueous] ====================
RunConfFit_Aqueous <- function(datawork = DATA.Aqueous.Work,
                               testparam = TEST.Aqueous.Design,
                               paramboot = BCF.ParamBoot,
                               savefit = TRUE,
                               steps   = 60,
                               tmin    = NULL,
                               tmax    = NULL,
                               textra  = NULL) {
  
  cwater <- testparam["cwater", "value"]
  tdepur <- testparam["tdepur", "value"]
  
  timesumm <- dataTime_Summary(datawork)
  
  if (is.null(tmin)) {
    tmin <- timesumm["min"]
  }
  
  if (is.null(tmax)) {
    tmax <- timesumm["max"]
  }
  
  if (!is.null(textra)) {
    trange <- tmax - tmin
    textend <- textra*trange
    
    tmin <- max(1e-9, tmin - textend)
    tmax <- tmax + textend
  }
  
  # -- time sequence supplemented with exact tdepur
  timepred <- sort(c(seq(tmin, tmax, length.out = steps + 1), tdepur))
  
  # -- how many boot instances
  lenboot <- length(row.names(paramboot))
  
  numtimes <- length(timepred)
  
  # -- declare space for conf estimates
  modconflo <- numeric(numtimes)
  modconfup <- numeric(numtimes)
  
  
  for (i in 1:numtimes) {
    
    timerepeat <- rep(timepred[i], lenboot)
    
    cfishboot <- RunModel_Aqueous(timerepeat, cwater, tdepur,
                                  paramboot$k1.boot, paramboot$k2.boot)
    
    modconflo[i] <- quantile(cfishboot, 0.025, type = 8)  # Tukey?
    modconfup[i] <- quantile(cfishboot, 0.975, type = 8)
    
  }
  
  # -- put into dataframe with appropriate names
  confframe <- data.frame(time.conf = timepred,
                          cfish.conflo = modconflo,
                          cfish.confup = modconfup)
  
  if (savefit)
    assign("BCF.ConfFit", confframe, envir = .GlobalEnv)
  
  top    <- head(confframe, n = 10)
  bottom <- tail(confframe, n = 10)
  
  print(top)
  cat("   ...")
  cat("\n")
  print(bottom)
  
  # return(confframe)
}
# ___ (RunConfFit_Aqueous) ____________________



# ////////////////////////////////////////////////
# === [RunConfFit_Aqueous_BoxCox] ====================
RunConfFit_Aqueous_BoxCox <- function(datawork = DATA.Aqueous.Work,
                                      testparam = TEST.Aqueous.Design,
                                      paramboot = BCF.ParamBoot.BoxCox,
                                      lambda    = BCF.ParamFit.Lambda,
                                      lnstarter = 1e-9,
                                      savefit = TRUE,
                                      steps   = 60,
                                      tmin    = NULL,
                                      tmax    = NULL,
                                      textra  = NULL) {
  
  cwater <- testparam["cwater", "value"]
  tdepur <- testparam["tdepur", "value"]
  
  timesumm <- dataTime_Summary(datawork)
  
  if (is.null(tmin)) {
    tmin <- timesumm["min"]
  }
  
  if (is.null(tmax)) {
    tmax <- timesumm["max"]
  }
  
  if (!is.null(textra)) {
    trange <- tmax - tmin
    textend <- textra*trange
    
    tmin <- max(1e-9, tmin - textend)
    tmax <- tmax + textend
  }
  
  # -- time sequence supplemented with exact tdepur
  timepred <- sort(c(seq(tmin, tmax, length.out = steps + 1), tdepur))
  
  # -- how many boot instances
  lenboot <- length(row.names(paramboot))
  
  numtimes <- length(timepred)
  
  # -- declare space for conf estimates: backtransformed
  invbcmodconflo <- numeric(numtimes)
  invbcmodconfup <- numeric(numtimes)
  
  # -- bc-transformed
  bcmodconflo <- numeric(numtimes)
  bcmodconfup <- numeric(numtimes)
  
  
  for (i in 1:numtimes) {
    
    timerepeat <- rep(timepred[i], lenboot)
    
    # -- untransformed modelrun on ln-transformed fitted params
    invbccfishboot <- RunModel_Aqueous(timerepeat, cwater, tdepur,
                                       paramboot$k1.boot.bc, paramboot$k2.boot.bc)
    
    bccfishboot <- car::bcPower(invbccfishboot + lnstarter, lambda)
    
    invbcmodconflo[i] <- quantile(invbccfishboot, 0.025, type = 8)
    invbcmodconfup[i] <- quantile(invbccfishboot, 0.975, type = 8)
    
    bcmodconflo[i] <- quantile(bccfishboot, 0.025, type = 8)
    bcmodconfup[i] <- quantile(bccfishboot, 0.975, type = 8)
    
  }
  
  # -- put into dataframe with appropriate names
  confframe <- data.frame(time.conf = timepred,
                          bc.cfish.conflo = bcmodconflo,
                          bc.cfish.confup = bcmodconfup,
                          inv.bc.cfish.conflo = invbcmodconflo,
                          inv.bc.cfish.confup = invbcmodconfup)
  
  if (savefit)
    assign("BCF.ConfFit.BoxCox", confframe, envir = .GlobalEnv)
  
  top    <- head(confframe, n = 10)
  bottom <- tail(confframe, n = 10)
  
  print(top)
  cat("   ...")
  cat("\n")
  print(bottom)
  
  # return(confframe)
}
# ___ (RunConfFit_Aqueous_BoxCox) ____________________


# ////////////////////////////////////////////////
# === [RunConfFit_Aqueous_Ln] ====================
RunConfFit_Aqueous_Ln <- function(datawork = DATA.Aqueous.Work,
                                  testparam = TEST.Aqueous.Design,
                                  paramboot = BCF.ParamBoot.Ln,
                                  lnstarter = 1e-9,
                                  savefit = TRUE,
                                  steps   = 60,
                                  tmin    = NULL,
                                  tmax    = NULL,
                                  textra  = NULL) {
  
  cwater <- testparam["cwater", "value"]
  tdepur <- testparam["tdepur", "value"]
  
  timesumm <- dataTime_Summary(datawork)
  
  if (is.null(tmin)) {
    tmin <- timesumm["min"]
  }
  
  if (is.null(tmax)) {
    tmax <- timesumm["max"]
  }
  
  if (!is.null(textra)) {
    trange <- tmax - tmin
    textend <- textra*trange
    
    tmin <- max(1e-9, tmin - textend)
    tmax <- tmax + textend
  }
  
  # -- time sequence supplemented with exact tdepur
  timepred <- sort(c(seq(tmin, tmax, length.out = steps + 1), tdepur))
  
  
  # -- how many boot instances
  lenboot <- length(row.names(paramboot))
  
  numtimes <- length(timepred)
  
  # -- declare space for conf estimates: backtransformed
  invlnmodconflo <- numeric(numtimes)
  invlnmodconfup <- numeric(numtimes)
  
  # -- ln-transformed
  lnmodconflo <- numeric(numtimes)
  lnmodconfup <- numeric(numtimes)
  
  
  for (i in 1:numtimes) {
    
    timerepeat <- rep(timepred[i], lenboot)
    
    # -- untransformed modelrun on ln-transformed fitted params
    invlncfishboot <- RunModel_Aqueous(timerepeat, cwater, tdepur,
                                       paramboot$k1.boot.ln, paramboot$k2.boot.ln)
    
    lncfishboot <- log(invlncfishboot + lnstarter)
    
    invlnmodconflo[i] <- quantile(invlncfishboot, 0.025, type = 8)
    invlnmodconfup[i] <- quantile(invlncfishboot, 0.975, type = 8)
    
    lnmodconflo[i] <- quantile(lncfishboot, 0.025, type = 8)
    lnmodconfup[i] <- quantile(lncfishboot, 0.975, type = 8)
    
  }
  
  # -- put into dataframe with appropriate names
  confframe <- data.frame(time.conf = timepred,
                          ln.cfish.conflo = lnmodconflo,
                          ln.cfish.confup = lnmodconfup,
                          inv.ln.cfish.conflo = invlnmodconflo,
                          inv.ln.cfish.confup = invlnmodconfup)
  
  if (savefit)
    assign("BCF.ConfFit.Ln", confframe, envir = .GlobalEnv)
  
  top    <- head(confframe, n = 10)
  bottom <- tail(confframe, n = 10)
  
  print(top)
  cat("   ...")
  cat("\n")
  print(bottom)
  
  # return(confframe)
}
# ___ (RunConfFit_Aqueous_Ln) ____________________


# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

# ///////////////////////////////////////////
# === [startFit_Aqueous] ====================
# --- FINAL bcmfR
StartFit_Aqueous <- function(datacfish = DATA.Aqueous.Work, 
                             testparam = TEST.Aqueous.Design,
                             lnstarter = 1e-9,
                             savefit   = TRUE) {
  
  testcwater <- testparam["cwater", "value"]
  timedepur  <- testparam["tdepur", "value"]
  
  #-- prepare the depuration data; include measurement at onset depur phase
  #-- yields a list of uptake & depur
  datasplit <- split_data_aqueous_at_tdepur(dataframe  = datacfish,
                                            timedepur  = timedepur,
                                            allocsplit = "both")
  
  #-- if savefit = TRUE, store phases also globally
  if (savefit) {
    
    # save the uptake and depuration parts
    assign("DATA.Aqueous.1.Uptake", datasplit$uptake, envir = .GlobalEnv)
    assign("DATA.Aqueous.2.Depur",  datasplit$depur,  envir = .GlobalEnv)
  }
  
  
  #-- fit linear model to the lndepur data
  linmodlndepur <- fit_linear_model_lndepur(datadepur = datasplit$depur,
                                            timedepur = timedepur,
                                            lnstarter = lnstarter)
  
  #-- fetch coefficients
  coefvec <- coef(linmodlndepur)

  # intercept
  beta0   <- coefvec[1]
  
  # k2, depuration rate constant is minus slope
  fitk2   <- -coefvec[2]
  fitk1   <- fitk2*exp(beta0)/testcwater/(exp(fitk2*timedepur) - 1)
  
  coefk1k2 <- c(fitk1, fitk2)
  
  names(coefk1k2) <- c("fitk1", "fitk2")
  
  return(coefk1k2)
  
}
# ___ (StartFit_Aqueous) ____________________



# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@


# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
# NEW SECTION ON RAW (UNTRANSFORMED) FITTING

# == Model.Fit.CFish.JointK1K2 =============================================
# -- fit 2-par nonlinear model on cfish; params: k1, k2
# -- uses nls()
Model.Fit.CFish.JointK1K2 <- function(datacfishln = Data.CFish.Ln, # KEEP
                                        testcwater  = Test.CWater.Value,
                                        timedepur   = Test.TDepur.Value,
                                        start       = NULL,
                                        trace       = FALSE,
                                        result      = "default") {
  
  # - available options
  optionvec = c("coef","coeferr","coefstats","confint","corrmat",
                "covarmat","deviance","default","loglik",
                "options","rss", "summary")
  
  # - return option vector, for result = "options"
  if (tolower(result) == "options") return(optionvec)
  
  # - if no start vector is given, calculate a guess from method Seq1
  if (is.null(start)) {
    start <- start.Fit.LnCFish.JointK1K2(datacfishln, testcwater, timedepur)
  } # KEEP LNCFISH STARTING
  
  # - nonlinear model fit on UNTRANSFORMED data
  modcfishjoint1 <- nls(var.cfish ~
                            modelCFishK1K2(var.time, testcwater,
                                               timedepur, fitk1, fitk2),
                          data  = datacfishln,
                          start = start,
                          trace = trace)
  
  modcfishjoint1
    
  coefmodcfishjoint1 <- coef(modcfishjoint1)
  
  deviancemodcfishjoint1 <- deviance(modcfishjoint1)
  
  summarycfishjoint1 <- summary(modcfishjoint1)
  
  coefstatscfishjoint1 <- coef(summary(modcfishjoint1)) # matrix
  
  varcovmodcfishjoint1 <- vcov(modcfishjoint1)
  
  corrmatcfishjoint1 <- varcovmodcfishjoint1
  
  #- diagonal
  corrmatcfishjoint1[1,1] <- 1.0
  corrmatcfishjoint1[2,2] <- 1.0
  
  #- off-diagonal
  corrmatcfishjoint1[1,2] <- varcovmodcfishjoint1[1,2]/
    coefstatscfishjoint1[1, 2]/
    coefstatscfishjoint1[2, 2]
  
  corrmatcfishjoint1[2,1] <- corrmatcfishjoint1[1,2]
  
  #- to avoid 'Waiting for profiling to be done message' only if necessary
  if(tolower(result) == "confint")
    confintmodcfishjoint1 <- confint(modcfishjoint1) # matrix
  
  #- to avoid 'Waiting for profiling to be done message' only if necessary
  if(tolower(result) == "coeferr") {
    
    coefestimatevec1  <- coefstatscfishjoint1[1:2, 1]
    coefstderrorvec1  <- coefstatscfishjoint1[1:2, 2]
    
    confintmodcfishjoint1 <- confint(modcfishjoint1)
    
    coefconfintlovec1 <- confintmodcfishjoint1[1:2, 1]
    coefconfintupvec1 <- confintmodcfishjoint1[1:2, 2]
    
    
    coeferrdataframe1 <- data.frame(coefestimatevec1,
                                    coefstderrorvec1,
                                    coefconfintlovec1,
                                    coefconfintupvec1)
    names(coeferrdataframe1) <- c("Estimate", "Std.Error","2.5%","97.5%")
    
  } # endif(tolower(result) == "coeferr")
  
  
  switch(tolower(result),
         #     c("coef","coeferr","coefstats","confint","corrmat","covarmat",
         #       "deviance","summary","default","options","rss")
         
         coef      = coefmodcfishjoint1,
         coeferr   = coeferrdataframe1,
         coefstats = coefstatscfishjoint1,
         confint   = confintmodcfishjoint1,
         corrmat   = corrmatcfishjoint1,
         covarmat  = varcovmodcfishjoint1,
         deviance  = deviancemodcfishjoint1,
         default    = modcfishjoint1,
         rss       = deviancemodcfishjoint1,
         summary   = summarycfishjoint1,
         
         stop("Not a valid result option."))
  
}
# == Model.Fit.CFish.JointK1K2 =====.


# == Fit.CFish.JointK1K2 =================================================
# -- 
Fit.CFish.JointK1K2 <- function(datacfishln = Data.CFish.Ln,
                                  testcwater  = Test.CWater.Value,
                                  timedepur   = Test.TDepur.Value,
                                  start       = NULL,
                                  trace       = FALSE) {
  
  Model.Fit.CFish.JointK1K2(datacfishln = datacfishln,
                              testcwater  = testcwater,
                              timedepur   = timedepur,
                              start       = start,
                              trace       = trace,
                              result      = "coef")
  
}
# == Fit.CFish.JointK1K2 =====.


#@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@.



# == start.Fit.LnCFish.JointBcfK2 ==========================================
# -- calculates starting values for Joint BcfK2 fit from Seq1 fit
# ++ silently assumes Data.CFish.Ln and so on; not correct if main
#     uses different data??

start.Fit.LnCFish.JointBcfK2 <- function(datacfishln = Data.CFish.Ln,
                                   testcwater  = Test.CWater.Value,
                                   timedepur   = Test.TDepur.Value) {
  
  # - fit linear depur
  # paralphabeta <- fit.statsLinearLnCFishDepur(datacfishln, timedepur)
  # startalpha = paralphabeta[[1]] # only for Seq3
  
  # - get k2 and bcf from fitBcfPar
  parfitseq1 <- Fit.BCFPar.LnCFish.Seq1(datacfishln, testcwater, timedepur)
  start_k2    <- parfitseq1[[3]]
  start_bcf   <- parfitseq1[[4]]
  
  # - nls needs a list
  start <- list(fitbcf = start_bcf, fitk2 = start_k2)
  
  # - return start
  start
  
}
# == start.Fit.LnCFish.JointBcfK2 =====.


# == Model.Fit.LnCFish.JointBcfK2 ==========================================
# -- fit 2-par nonlinear model on lncfish; params: k1, k2
# -- uses nls()

Model.Fit.LnCFish.JointBcfK2 <- function(datacfishln = Data.CFish.Ln,
                                   testcwater  = Test.CWater.Value,
                                   timedepur   = Test.TDepur.Value,
                                   start       = NULL,
                                   trace       = FALSE,
                                   result      = "default") {
  
  # - available options
  optionvec = c("coef","coeferr","coefstats","confint",
                "covarmat","deviance","summary","default",
                "options","rss")
  
  # - return option vector, for result = "options"
  if (tolower(result) == "options") return(optionvec)
  
  if (is.null(start)) {
    start <- start.Fit.LnCFish.JointBcfK2(datacfishln, testcwater, timedepur)
  }
  
  modlncfishjoint2 <- nls(var.lncfish ~
                          log(modelCFishBcfK2(var.time, testcwater, timedepur,
                                                fitbcf, fitk2)),
                          data  = datacfishln,
                          start = start,
                          trace = trace)
  
  deviancemodlncfishjoint2 <- deviance(modlncfishjoint2)
  
  summarylncfishjoint2 <- summary(modlncfishjoint2)
  
  coefmodlncfishjoint2 <- coef(modlncfishjoint2)
  
  coefstatslncfishjoint2 <- coef(summary(modlncfishjoint2)) # matrix
  
  varcovmodlncfishjoint2 <- vcov(modlncfishjoint2)
  
  corrmatlncfishjoint2 <- varcovmodlncfishjoint2
  # diagonal
  corrmatlncfishjoint2[1,1] <- 1.0
  corrmatlncfishjoint2[2,2] <- 1.0
  # off-diagonal
  corrmatlncfishjoint2[1,2] <- varcovmodlncfishjoint2[1,2]/
    coefstatslncfishjoint2[1, 2]/coefstatslncfishjoint2[2, 2]
  
  corrmatlncfishjoint2[2,1] <- corrmatlncfishjoint2[1,2]
  
  # -- to avoid 'Waiting for profiling to be done message' only if necessary
  if(tolower(result) == "confint")
    confintmodlncfishjoint2 <- confint(modlncfishjoint2) # matrix
  
  # -- to avoid 'Waiting for profiling to be done message' only if necessary
  if(tolower(result) == "coeferr") {
    
    coefestimatevec2  <- coefstatslncfishjoint2[1:2, 1]
    coefstderrorvec2  <- coefstatslncfishjoint2[1:2, 2]
    
    confintmodlncfishjoint2 <- confint(modlncfishjoint2)
    
    coefconfintlovec2 <- confintmodlncfishjoint2[1:2, 1]
    coefconfintupvec2 <- confintmodlncfishjoint2[1:2, 2]
    
    
    coeferrdataframe2 <- data.frame(coefestimatevec2,
                                      coefstderrorvec2,
                                      coefconfintlovec2,
                                      coefconfintupvec2)
    names(coeferrdataframe2) <- c("Estimate", "Std.Error","2.5%","97.5%")
  }
  
  switch(tolower(result),
         #          c("coef","coeferr","coefstats","confint","corrmat",
         #            "covarmat","deviance","default","options",
         #            "rss","summary")
         
         coef      = coefmodlncfishjoint2,
         coeferr   = coeferrdataframe2,
         coefstats = coefstatslncfishjoint2,
         confint   = confintmodlncfishjoint2,
         corrmat   = corrmatlncfishjoint2,
         covarmat  = varcovmodlncfishjoint2,
         deviance  = deviancemodlncfishjoint2,
         default     = modlncfishjoint2,
         rss       = deviancemodlncfishjoint2,
         summary   = summarylncfishjoint2,
         
         stop("Not a valid result option."))
  
}
# == Model.Fit.LnCFish.JointBcfK2 =====.


# == Fit.LnCFish.JointBcfK2 ================================================
# --

Fit.LnCFish.JointBcfK2 <- function(datacfishln = Data.CFish.Ln,
                                  testcwater  = Test.CWater.Value,
                                  timedepur   = Test.TDepur.Value,
                                  start       = NULL,
                                  trace       = FALSE) {
  
  Model.Fit.LnCFish.JointBcfK2(datacfishln = datacfishln,
                         testcwater = testcwater,
                         timedepur = timedepur,
                         start  = start,
                         trace  = trace,
                         result = "coef")

}
# == Fit.LnCFish.JointBcfK2 =====.



#$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
# NEW SECTION FOR RAW CFISH FITTING WITH PARAMS BCF AND K2

# == start.Fit.CFish.JointBcfK2 ============================================
# -- calculates starting values for Joint BcfK2 fit from Seq1 fit
# ++ silently assumes Data.CFish.Ln and so on; not correct if main
#     uses different data??

start.Fit.CFish.JointBcfK2 <- function(datacfishln = Data.CFish.Ln,
                                         testcwater  = Test.CWater.Value,
                                         timedepur   = Test.TDepur.Value) {
  
  # - fit linear depur
  # paralphabeta <- fit.statsLinearLnCFishDepur(datacfishln, timedepur)
  # startalpha = paralphabeta[[1]] # only for Seq3
  
  # - get k2 and bcf from fitBcfPar
  parfitseq1 <- Fit.BCFPar.LnCFish.Seq1(datacfishln, testcwater, timedepur)
  start_k2    <- parfitseq1[[3]]
  start_bcf   <- parfitseq1[[4]]
  
  # - nls needs a list
  start <- list(fitbcf = start_bcf, fitk2 = start_k2)
  
  # - return start
  start
  
}
# == start.Fit.CFish.JointBcfK2 =====.


# == Model.Fit.CFish.JointBcfK2 ============================================
# -- fit 2-par nonlinear model on lncfish; params: k1, k2
# -- uses nls()

Model.Fit.CFish.JointBcfK2 <- function(datacfishln = Data.CFish.Ln, # KEEP
                                         testcwater  = Test.CWater.Value,
                                         timedepur   = Test.TDepur.Value,
                                         start       = NULL,
                                         trace       = FALSE,
                                         result      = "default") {
  
  # - available options
  optionvec = c("coef","coeferr","coefstats","confint",
                "covarmat","deviance","summary","default",
                "options","rss")
  
  # - return option vector, for result = "options"
  if (tolower(result) == "options") return(optionvec)
  
  if (is.null(start)) {
    start <- start.Fit.LnCFish.JointBcfK2(datacfishln, testcwater, timedepur)
  } # KEEP LN FITTING FOR START
  
  modcfishjoint2 <- nls(var.cfish ~
                            modelCFishBcfK2(var.time, testcwater, timedepur,
                                                fitbcf, fitk2),
                          data  = datacfishln,
                          start = start,
                          trace = trace)
  
  deviancemodcfishjoint2 <- deviance(modcfishjoint2)
  
  summarycfishjoint2 <- summary(modcfishjoint2)
  
  coefmodcfishjoint2 <- coef(modcfishjoint2)
  
  coefstatscfishjoint2 <- coef(summary(modcfishjoint2)) # matrix
  
  varcovmodcfishjoint2 <- vcov(modcfishjoint2)
  
  corrmatcfishjoint2 <- varcovmodcfishjoint2
  # diagonal
  corrmatcfishjoint2[1,1] <- 1.0
  corrmatcfishjoint2[2,2] <- 1.0
  # off-diagonal
  corrmatcfishjoint2[1,2] <- varcovmodcfishjoint2[1,2]/
    coefstatscfishjoint2[1, 2]/coefstatscfishjoint2[2, 2]
  
  corrmatcfishjoint2[2,1] <- corrmatcfishjoint2[1,2]
  
  # -- to avoid 'Waiting for profiling to be done message' only if necessary
  if(tolower(result) == "confint")
    confintmodcfishjoint2 <- confint(modcfishjoint2) # matrix
  
  # -- to avoid 'Waiting for profiling to be done message' only if necessary
  if(tolower(result) == "coeferr") {
    
    coefestimatevec2  <- coefstatscfishjoint2[1:2, 1]
    coefstderrorvec2  <- coefstatscfishjoint2[1:2, 2]
    
    confintmodcfishjoint2 <- confint(modcfishjoint2)
    
    coefconfintlovec2 <- confintmodcfishjoint2[1:2, 1]
    coefconfintupvec2 <- confintmodcfishjoint2[1:2, 2]
    
    
    coeferrdataframe2 <- data.frame(coefestimatevec2,
                                    coefstderrorvec2,
                                    coefconfintlovec2,
                                    coefconfintupvec2)
    names(coeferrdataframe2) <- c("Estimate", "Std.Error","2.5%","97.5%")
  }
  
  switch(tolower(result),
         #          c("coef","coeferr","coefstats","confint","corrmat",
         #            "covarmat","deviance","default","options",
         #            "rss","summary")
         
         coef      = coefmodcfishjoint2,
         coeferr   = coeferrdataframe2,
         coefstats = coefstatscfishjoint2,
         confint   = confintmodcfishjoint2,
         corrmat   = corrmatcfishjoint2,
         covarmat  = varcovmodcfishjoint2,
         deviance  = deviancemodcfishjoint2,
         default    = modcfishjoint2,
         rss       = deviancemodcfishjoint2,
         summary   = summarycfishjoint2,
         
         stop("Not a valid result option."))
  
}
# == Model.Fit.CFish.JointBcfK2 =====.


# == Fit.LnCFish.JointBcfK2 =================================================
# --

Fit.CFish.JointBcfK2 <- function(datacfishln = Data.CFish.Ln,
                                 testcwater  = Test.CWater.Value,
                                 timedepur   = Test.TDepur.Value,
                                 start       = NULL,
                                 trace       = FALSE) {
  
  Model.Fit.CFish.JointBcfK2(datacfishln = datacfishln,
                               testcwater = testcwater,
                               timedepur = timedepur,
                               start  = start,
                               trace  = trace,
                               result = "coef")
  
}
# == Fit.CFish.JointBcfK2 =====.

#$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$.


# == Fit.BCFPar.LnCFish.Joint ==============================================
# ++ argument datasets
Fit.BCFPar.LnCFish.Joint <- function() {
  
  bestk1 <- Fit.LnCFish.JointK1K2()[[1]]
  
  bestk2 <- Fit.LnCFish.JointK1K2()[[2]]
  
  bestbcf <- Fit.LnCFish.JointBcfK2()[[1]]
  
  fitout <- c(bestk1, bestk2, bestbcf)
  
  names(fitout) <- c("k1","k2","bcf")
  
  fitout  
  
}
# == Fit.BCFPar.LnCFish.Joint =====.


# == Stats.BCFPar.LnCFish.Joint ============================================
Stats.BCFPar.LnCFish.Joint <- function(digits = NULL) {
  
  #-- package lucid for clear print (check library vs. require)
  require(lucid)
  
  #- gives data frame
  estk1k2 <- Model.Fit.LnCFish.JointK1K2(result = "coeferr")
  
  #- gives data frame
  estbcfk2 <- Model.Fit.LnCFish.JointBcfK2(result = "coeferr")
  
  #- take k1, k2 estimates; add bcf estimate to form a vector
  colestimate <- c(estk1k2$Estimate, estbcfk2$Estimate[1])

  #- k1, k2 standard errors; add bcf standard error to form a vector
  colstderr <- c(estk1k2$Std.Error, estbcfk2$Std.Error[1])
  
  #- same here; note the backquotes to name the 2.5% field
  colconflo <- c(estk1k2$`2.5%`, estbcfk2$`2.5%`[1])
  
  #- similar for upper confidence limits
  colconfup <- c(estk1k2$`97.5%`, estbcfk2$`97.5%`[1])
  
  #- build output data frame
  estdf <- data.frame(colestimate, colstderr, colconflo, colconfup)  
  
  #- set header names
  names(estdf) <- c("Estimate", "Std.Error", "2.5%", "97.5%")
  
  #- set row names
  row.names(estdf) <- c("k1", "k2", "bcf")
  
  #- store in global BCFPar.CFish.Joint
  BCFPar.LnCFish.Joint <<- estdf
  
  #- return estdf
  if (is.null(digits)) {
    estdf
  } else {
    lucid(estdf, dig=digits)
  }
}
# == Stats.BCFPar.LnCFish.Joint =====.


#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

# == Fit.BCFPar.CFish.Joint ================================================
# ++ argument datasets
Fit.BCFPar.CFish.Joint <- function() {
  
  bestk1 <- Fit.CFish.JointK1K2()[[1]]
  
  bestk2 <- Fit.CFish.JointK1K2()[[2]]
  
  bestbcf <- Fit.CFish.JointBcfK2()[[1]]
  
  fitout <- c(bestk1, bestk2, bestbcf)
  
  names(fitout) <- c("k1","k2","bcf")
  
  fitout  
  
}
# == Fit.BCFPar.CFish.Joint =====.


# == Stats.BCFPar.LnCFish.Joint ============================================
Stats.BCFPar.CFish.Joint <- function(digits = NULL) {
  
  #-- package lucid for clear print (check library vs. require)
  require(lucid)
  
  #- gives data frame
  estk1k2 <- Model.Fit.CFish.JointK1K2(result = "coeferr")
  
  #- gives data frame
  estbcfk2 <- Model.Fit.CFish.JointBcfK2(result = "coeferr")
  
  #- take k1, k2 estimates; add bcf estimate to form a vector
  colestimate <- c(estk1k2$Estimate, estbcfk2$Estimate[1])
  
  #- k1, k2 standard errors; add bcf standard error to form a vector
  colstderr <- c(estk1k2$Std.Error, estbcfk2$Std.Error[1])
  
  #- same here; note the backquotes to name the 2.5% field
  colconflo <- c(estk1k2$`2.5%`, estbcfk2$`2.5%`[1])
  
  #- similar for upper confidence limits
  colconfup <- c(estk1k2$`97.5%`, estbcfk2$`97.5%`[1])
  
  #- build output data frame
  estdf <- data.frame(colestimate, colstderr, colconflo, colconfup)  
  
  #- set header names
  names(estdf) <- c("Estimate", "Std.Error", "2.5%", "97.5%")
  
  #- set row names
  row.names(estdf) <- c("k1", "k2", "bcf")
  
  #- store in global BCFPar.CFish.Joint
  BCFPar.CFish.Joint <<- estdf
  
  #- return estdf
  if (is.null(digits)) {
    estdf
  } else {
    lucid(estdf, dig=digits)
  }
}
# == Stats.BCFPar.CFish.Joint =====.

# _________________________________________________________________________
# %% END OF R-SOURCE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%