# %% bcmfR 0.3-1 R-source %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# __________________________________________________________________________
#
#    bcmfR-bcfseq1_17.R
# __________________________________________________________________________



# === [split_data_aqueous_at_tdepur] ======================
# --- FINAL bcmfR helper function
# --- split dataframe into uptake and depuration parts
# --- assumes name "time.data"
split_data_aqueous_at_tdepur <- function(dataframe,
                                         timedepur,
                                         allocsplit = "both") {
  # "uptake", "depur", "neither"
  
  # -- split at timedepur; allocsplit determines allocation at timedepur
  switch(allocsplit,
         both = {
           datadepur  <- subset(dataframe, time.data >= timedepur)
           datauptake <- subset(dataframe, time.data <= timedepur)
         },
         uptake = {
           datadepur  <- subset(dataframe, time.data >  timedepur)
           datauptake <- subset(dataframe, time.data <= timedepur)
         },
         depur = {
           datadepur  <- subset(dataframe, time.data >= timedepur)
           datauptake <- subset(dataframe, time.data < timedepur)
         },
         neither = {
           datadepur  <- subset(dataframe, time.data > timedepur)
           datauptake <- subset(dataframe, time.data < timedepur)
         },
         stop("Invalid allocsplit option"))
  
  list(uptake = datauptake, depur = datadepur, alloc.split = allocsplit)
  
}
# ___ (split_data_aqueous_at_tdepur) ____________________



# === [fit_linear_model_lndepur] ============
# --- FINAL bcmfR: helper function
fit_linear_model_lndepur <- function(datadepur,
                                     timedepur,
                                     lnstarter = 1e-9) {
  
  #-- prepare the depuration data; include measurement at onset depur phase
  # splitdatacfish <- split_data_aqueous_at_tdepur(dataframe = datacfish,
  #                                               timedepur = timedepur)
  
  #-- fit a linear model to ln(cfish)                                  
  linmodlndepur <- lm(log(cfish.data + lnstarter) ~ time.data,
                        data = datadepur)
  
  # splitandlinmod <- list(splitdata = splitdatacfish,
  #                        lmlndepur = linmodlndepur)
 
  return(linmodlndepur)
  
}
# ___ (fit_linear_model_lndepur) _____________



# === [parErr_Model_BCFK1K2_LnDepur] ====================
# --- FINAL bcmfR
parErr_Model_BCFK1K2_LnDepur <- function(datacfish = DATA.Aqueous.Work,
                                         testparam = TEST.Aqueous.Param,
                                         digits    = NULL,
                                         lnstarter = 1e-9,
                                         savefit   = TRUE) {

  testcwater <- testparam["cwater"]
  timedepur  <- testparam["tdepur"]

  #-- prepare the depuration data; include measurement at onset depur phase
  #-- yields a list of uptake & depur
  datasplit <- split_data_aqueous_at_tdepur(dataframe  = datacfish,
                                            timedepur  = timedepur,
                                            allocsplit = "both")

  #-- if globaldata = TRUE, store phases also globally
  if (savefit) {
    assign("DATA.Aqueous.2.Depur",  datasplit$depur,  envir = .GlobalEnv)
    assign("DATA.Aqueous.1.Uptake", datasplit$uptake, envir = .GlobalEnv)
  }
  
    
  #-- fit linear model to the lndepur data
  linmodlndepur <- fit_linear_model_lndepur(datadepur = datasplit$depur,
                                            timedepur = timedepur,
                                            lnstarter = lnstarter)
  
  # splitdatacfish <- splitandlinmod$splitdata
  # linmodlndepur  <- splitandlinmod$lmlndepur

  
  parnames <- c("beta0","beta1")
  
  #-- same as for bcf, but multiplied by k2 = - beta1
  gfunck1 <- paste("-beta1*exp(beta0)/", testcwater,
                   "/(exp(-beta1*", timedepur, ") - 1)", sep = "")
  
  #-- gamma
  gfuncgamma <- paste("beta0 + beta1*", timedepur, sep = "")
  
  #-- k2 is just minus beta1
  gfunck2 <- "-beta1"
  
  #-- our elimination of bcf from beta0
  gfuncbcf <- paste("exp(beta0)/", testcwater,
                    "/(exp(-beta1*", timedepur, ") - 1)", sep = "")
  
  #-- deltaMethod from package car for k1
  delmetk1  <- deltaMethod(linmodlndepur, g = gfunck1,
                           parameterNames = parnames)
  
  #-- deltaMethod from package car for gamma
  delmetgamma  <- deltaMethod(linmodlndepur, g = gfuncgamma,
                              parameterNames = parnames)
  
  #-- deltaMethod from package car for k2
  delmetk2  <- deltaMethod(linmodlndepur, g = gfunck2,
                           parameterNames = parnames)
  
  #-- deltaMethod from package car for bcf
  delmetbcf <- deltaMethod(linmodlndepur, g = gfuncbcf,
                           parameterNames = parnames)
  
  #-- collect individual estimates and standard errors
  estk1 <- delmetk1[[1]]
  estgamma <- delmetgamma[[1]]
  estk2 <- delmetk2[[1]]
  estbcf <- delmetbcf[[1]]
  
  #-- and standard errors
  errk1 <- delmetk1[[2]]
  errgamma <- delmetgamma[[2]]
  errk2 <- delmetk2[[2]]
  errbcf <- delmetbcf[[2]]
  
  #-- build Wald confidence intervals
  conflok1 <- estk1 - 1.96*errk1
  confupk1 <- estk1 + 1.96*errk1
  
  conflogamma <- estgamma - 1.96*errgamma
  confupgamma <- estgamma + 1.96*errgamma
  
  conflok2 <- estk2 - 1.96*errk2
  confupk2 <- estk2 + 1.96*errk2
  
  conflobcf <- estbcf - 1.96*errbcf
  confupbcf <- estbcf + 1.96*errbcf
  
  #-- put the values in a data frame
  dfk1k2bcf <- data.frame(c(estk1, estgamma, estk2, estbcf),
                          c(errk1, errgamma, errk2, errbcf),
                          c(conflok1, conflogamma, conflok2, conflobcf),
                          c(confupk1, confupgamma, confupk2, confupbcf))
  
  #-- set header names
  names(dfk1k2bcf) <- c("Estimate", "Std.Error", "2.5%", "97.5%")
  
  #-- set parameter row names
  row.names(dfk1k2bcf) <- c("k1", "gamma", "k2", "bcf")
  
  # prepare for parameter estimate (first column)
  parest <- dfk1k2bcf[[1]]
  names(parest) <- row.names(dfk1k2bcf)

  if (savefit) { 
    #-- store point estimate
    assign("BCF.ModelFit.LnDepur", parest, envir = .GlobalEnv)
  
    #-- store uncertainty frame in global BCFPar.CFish.Joint
    assign("BCF.ParamErr.LnDepur", dfk1k2bcf, envir = .GlobalEnv)
  }
  
  #-- return estimation dataframe
  if (is.null(digits))
    dfk1k2bcf
  else
    lucid(dfk1k2bcf, dig = digits)
  
}
# ___ (parErr_Model_BCFK1K2_LnDepur) ____________________



#=== Model.Fit.LnCFish.Depur ================================================#
#--- R model object, and many statistical result options

#+++ provide for nonlinear fit, without log transform?

Model.Fit.LnCFish.Depur <- function(datacfishln = Data.CFish.Ln,
                                  timedepur   = Test.TDepur.Value,
                                  result      = "object") {
  
  #-- available options
  optionvec <- c("coef","coeferr","coefstats","confint",
                 "covarmat","deviance","summary","object","options")
  
  #-- return option vector, for result = "options"
  if (tolower(result) == "options") return(optionvec)

  
  #-- prepare the depuration data; include measurement at onset depur phase
  splitdatacfishln <- split.Data.At.TDepur(dataframe = datacfishln,
                                           timedepur = timedepur)
  
  #-- store also as global data
  Data.CFish.Ln.Depur  <<- splitdatacfishln$data.depur
  Data.CFish.Ln.Uptake <<- splitdatacfishln$data.uptake
  
  #-- fit a linear model to lncfish                                  
  modlncfishdepur <- lm(var.lncfish ~ var.time, data = Data.CFish.Ln.Depur)
  
  coefvec <- coef(modlncfishdepur) # class "numeric"
  names(coefvec) <- c("beta0", "beta1")
  
  summarylm <- summary(modlncfishdepur)
  
  #-- extract the coefficient estimate and rename to beta0 and beta1
  coefsummarymat <- coef(summary(modlncfishdepur)) # class "matrix"
  row.names(coefsummarymat) <- c("beta0", "beta1")
  
  #-- confidence intervals
  coefconfintmat <- confint(modlncfishdepur) # class "matrix" (Mass)
  row.names(coefconfintmat) <- c("beta0", "beta1")
  
  coefestimatevec  <- coefsummarymat[1:2, 1]
  coefstderrorvec  <- coefsummarymat[1:2, 2]
  coefconfintlovec <- coefconfintmat[1:2, 1]
  coefconfintupvec <- coefconfintmat[1:2, 2]
  
  coefestimatedataframe <- data.frame(coefestimatevec)
  names(coefestimatedataframe) <- c("Estimate")
  
  coeferrordataframe <- data.frame(coefestimatevec,
                                coefstderrorvec,
                                coefconfintlovec,
                                coefconfintupvec)
  
  names(coeferrordataframe) <- c("Estimate", "Std.Error", "2.5%", "97.5%")
  
    
  switch(tolower(result),
         
         coef        = coefvec,
         coeferr     = coeferrordataframe,
         coefstats   = coefsummarymat,
         confint     = coefconfintmat,
         # corrmat
         covarmat    = vcov(modlncfishdepur),
         deviance    = deviance(modlncfishdepur),
         object      = modlncfishdepur,
         rss         = deviance(modlncfishdepur),
         summary     = summarylm,
                 
         stop("Not a valid result option."))
  
}
#=== Model.Fit.LnCFish.Depur =====.


#=== Fit.LnCFish.Depur =================================================#
#--- shortcut to parameter point estimate (vector)

Fit.LnCFish.Depur <- function(datacfishln = Data.CFish.Ln,
                                 timedepur   = Test.TDepur.Value) {
  
  #-- call the main model routine with result = "coef"
  Model.Fit.LnCFish.Depur(datacfishln = datacfishln,
                        timedepur   = timedepur,
                        result      = "coef")
  
}
#=== Fit.LnCFish.Depur ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~.


#=== Fit.LnCFish.Depur ==============================================#
#--- shortcut to parameter estimation table

Fit.LnCFish.Depur <- function(datacfishln = Data.CFish.Ln,
                                  timedepur   = Test.TDepur.Value) {
  
  #- call the main model routine with result = "coeferr"
  Model.Fit.LnCFish.Depur(datacfishln = datacfishln,
                        timedepur   = timedepur,
                        result = "coeferr")
  
}
#=== Fit.LnCFish.Depur ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~.






#=== Fit.BCFPar.LnCFish.Seq1 ================================================#
#--- bcf parameters: k1, k2, bcf as point estimates for sequential #1 

Fit.BCFPar.LnCFish.Seq1 <- function (datacfishln = Data.CFish.Ln,
                                   testcwater  = Test.CWater.Value,
                                   timedepur   = Test.TDepur.Value,
                                   digits      = NULL) {
                                 # default method = "delta".

  #-- calc full summary table of parameter statss
  statsk1k2bcfdataframe <- Stats.BCFPar.LnCFish.Seq1(datacfishln,
                                                   testcwater, timedepur)
  
  #-- take the vector of point estimates: first column
  park1k2bcfestvec <- statsk1k2bcfdataframe[[1]]
  
  #-- attach names to the elements
  names(park1k2bcfestvec) <- c("k1", "gamma", "k2", "bcf")
  
  #-- return estimation dataframe
  if(is.null(digits)) {
    park1k2bcfestvec
  } else {
    noquote(str_trim(lucid(park1k2bcfestvec, dig=digits)))
  }
}
#=== Fit.BCFPar.LnCFish.Seq1 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~.


# #=== Stats.BCFPar.LnCFish.Seq1.ParK2 ========================================#
# #---
# 
# Stats.BCFPar.LnCFish.Seq1.ParK2 <- function(datacfishln = Data.CFish.Ln,
#                                   testcwater  = Test.CWater.Value,
#                                   timedepur   = Test.TDepur.Value,
#                                   method      = "delta") { # no other method
#   
#   
#   #-- call Stats.BCFPar.LnCFish.Seq1
#   bcfpartable <- Stats.BCFPar.LnCFish.Seq1(datacfishln = datacfishln,
#                                         testcwater  = testcwater,
#                                         timedepur   = timedepur,
#                                         method      = method)
#   
#   #-- return the k2 data frame
#   bcfpartable["k2",]
#   
# }
# #=== Stats.BCFPar.LnCFish.Seq1.ParK2 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~.


# %% END OF R-SOURCE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%